#ifndef GEOMETRIA_BASICA_H
#define GEOMETRIA_BASICA_H

#include <CGAL/Cartesian.h>
#include <CGAL/Triangulation_euclidean_traits_xy_3.h>
#include <CGAL/Triangulation_face_base_with_info_2.h>
#include <CGAL/Triangulation_vertex_base_with_info_2.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <qcolor.h>
#include <vector.h>
#include <list.h>

#define PI                3.1415926535897932    
#define TOLERANCE         1e-13 
#define SKINNY            20.7
#define REFINE_ANGLE      20.7   

#define VOLUM_INTERSECCIO 0
#define VOLUM_DELAUNAY    1
#define VOLUM_CORRECTE    2


enum ElementsVisio { INDEFINIT, PUNT_VISIO, SEGMENT_VISIO, SEGMENT_TERRENY, POLIGON_VISIO, POLIGONAL_VISIO };

enum TipusPoligon { POLIGON_IRREGULAR, POLIGON_CONVEX };

enum TipusVisualitzacio { UNIO, INTERSECCIO , COUNTING , SEGMENT , SEGMENT_CANVIAT , COMBINAT };

enum Events { EVENT_FINESTRA, EVENT_PSEUDOSOURCE };

/*******************************************************************************************/
/*                                Definicions de Tipus                                     */
/*   Fitxer que engloba definicions de diferents tipus utilitzats en les classes creades   */
/*******************************************************************************************/

typedef CGAL::Cartesian<double>  Rep;

typedef CGAL::Point_3<Rep>   	 Punt3d;
typedef CGAL::Point_2<Rep>   	 Punt2d;
typedef CGAL::Vector_3<Rep>  	 Vector;
typedef CGAL::Vector_2<Rep>  	 Vector2D;
typedef CGAL::Line_3<Rep>    	 Line;
typedef CGAL::Line_2<Rep>    	 Line2D;
typedef CGAL::Triangle_3<Rep>  	 Triangle3;
typedef CGAL::Triangle_2<Rep>  	 Triangle2;
typedef CGAL::Plane_3<Rep>  	 Plane;// Redefinició d'operadors
typedef CGAL::Segment_3<Rep>  	 Segment3D;
typedef CGAL::Segment_2<Rep>  	 Segment2D;
typedef CGAL::Tetrahedron_3<Rep> Tetrahedron;
typedef CGAL::Aff_transformation_2<Rep> Transformation2;
typedef CGAL::Aff_transformation_3<Rep> Transformation3;

class Finestra;
//class PseudoSource;


inline double gr2rd( double graus )
{
   return( graus * PI / 180.0 );
}

inline double rd2gr( double radians )
{
   return( radians * 180.0 / PI );
}

inline void normalitza( Vector &v )
{
   double longitud = sqrt( v * v );   
   v = v / longitud;
}

inline bool iguals( double a, double b )
{
   return ((fabs(a-b) < 0.0000000001) ? TRUE : FALSE);
}

inline bool mesGran( double a, double b )
{
   return ((a > b) && !iguals(a,b));
}

inline bool mesPetit( double a, double b )
{
   return ((a < b) && !iguals(a,b));
}

struct Cara
{ 
   Triangle3 t;
   QColor color;
   double nivell;
   Vector normal;
   
   Cara()
   {
      t = Triangle3( Punt3d(0.0, 0.0, 0.0), Punt3d(0.0, 0.0, 0.0), Punt3d(0.0, 0.0, 0.0) );
      color = QColor(170, 85, 0);
      nivell = 0;
      normal = Vector(0.0, 0.0, 0.0);
   }

   Cara( Punt3d p0, Punt3d p1, Punt3d p2, QColor c = QColor(170, 85, 0) )
   {
      t = Triangle3( p0, p1, p2 );
      color = c;
      nivell = 0;

      Vector vab = t[1] - t[0]; // vector AB 
      Vector vac = t[2] - t[0]; // vector AC 
      normal = CGAL::cross_product( vab, vac );   //  AB ^ AC
      normalitza(normal); 
   }
   
   bool operator ==( Cara &a )
   {
       return( t == a.t );
   }
   
   bool operator !=( Cara &a )
   {
       return !( *this == a );
   }
   
   Punt3d operator[]( int idx )
   {
      assert( idx >= 0 && idx <= 2 );
      return t[idx];
   }

   Punt3d vertex( int idx )
   {
      assert( idx >= 0 && idx <= 2 );
      return t[idx];
   }
   
   bool teVertex( Punt3d p )
   {
      return( (*this)[0] == p || (*this)[1] == p || (*this)[2] == p );
   }
};

typedef std::list<Cara>   LlistaCares;
typedef std::list<Punt3d> LlistaPunts;

//necessitarem una altra priority_queue (list) de events, que s'haurà d'ordenar per pes. 
//PER EVENTS < i que ordenes per pes

struct InfoFace
{
   LlistaCares cares;
   Vector control[7];
   Vector v_normals[3];
   double nivell;
   Vector normal;
   double entrada[3][4];
   bool entradaCalculada;
   Transformation3 T[3];
   std::list<Finestra> LlistaFinestres[3];
};


struct InfoVertex
{
   double distancia;
   int index;
};

struct Corba
{
    LlistaPunts punts;
    QColor color;
};

typedef std::list<Corba>  LlistaCorbes;

typedef CGAL::Triangulation_euclidean_traits_xy_3<Rep>             Gt;
//typedef CGAL::Triangulation_vertex_base_2<Gt>                      Vb;
typedef CGAL::Triangulation_vertex_base_with_info_2<InfoVertex, Gt>  Vb;
typedef CGAL::Triangulation_face_base_with_info_2<InfoFace, Gt>    Fb;
typedef CGAL::Triangulation_data_structure_2<Vb,Fb>                Tds;
typedef CGAL::Delaunay_triangulation_2<Gt,Tds>                     Delaunay;

typedef Delaunay::Finite_faces_iterator 	IteradorCares;
typedef Delaunay::Finite_vertices_iterator 	IteradorVertexs;
typedef Delaunay::Point_iterator                IteradorPunts;
typedef Delaunay::Vertex_handle 		pVertex;
typedef Delaunay::Face_handle 			pCara;
typedef Delaunay::Vertex_circulator 		CirculadorVertexs;
typedef Delaunay::Face_circulator 		CirculadorCares;
typedef Delaunay::Locate_type 			Localitzacio;
typedef CGAL::Oriented_side			Orientacio;

#endif   //GEOMETRIA_BASICA_H
